/*
** init_spi.c                          
**
** Initialize SPI programmatically
**
** Copyright (c) 2011-2012 Analog Devices, Inc. All Rights Reserved.
**/

#include "init_spi.h"

__ramfunc static void spi_configure(ADI_SPI_TypeDef *pSpiRegs);
__ramfunc static void spi_regs_copy(ADI_SPI_TypeDef *pSrc, ADI_SPI_TypeDef *pDest);
__ramfunc static void spi_send_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t * pData, uint32_t nNumBytes);
__ramfunc static void spi_exit_xip(ADI_SPI_TypeDef * pSpiRegs);
__ramfunc static void spi_enter_xip(ADI_SPI_TypeDef * pSpiRegs);

int temp=0;
unsigned int* pSPI_MEM = (unsigned int*) 0x18004000;

__ramfunc INIT_SPI_RESULT init_spi(void)
{    
  ADI_SPI_TypeDef *pSPI = pADI_SPI2;
  ADI_SPI_TypeDef spiBackup;

  pSPI->CTL &= ~BITM_SPI_CTL_EN;         /* Disable the SPI before reconfiguring */
  
  spi_regs_copy(pSPI, &spiBackup);       /* Backup our SPI registers */
  spi_configure(pSPI);                   /* Put the SPI into single bit mode */

  spi_exit_xip(pSPI);   /* Exit the XIP mode by issuing 0xFF command */
 
  spi_regs_copy(&spiBackup, pSPI);  /* Put back into quad mode and memory mapped mode  */
  
  *pREG_M4P0_CACHE_CFG |= ((7 << BITP_M4P_CACHE_CFG_PF ) & BITM_M4P_CACHE_CFG_PF );     /* Enabling prefetch feature of cache */
  
  pSPI->CLK = 0; /* Increase the speed of SPI such that SPICLK = SCLK */
  
  spi_enter_xip(pSPI);
  
return INIT_SPI_RESULT_SUCCESS; 
}


__ramfunc static void spi_regs_copy(ADI_SPI_TypeDef *pSrc, ADI_SPI_TypeDef *pDest)
{
  pDest->CTL = pSrc->CTL & ~BITM_SPI_CTL_EN; /* Backup but ensure that SPI is disabled */
  pDest->RXCTL = pSrc->RXCTL;
  pDest->TXCTL = pSrc->TXCTL;
  pDest->DLY = pSrc->DLY;
  pDest->SLVSEL = pSrc->SLVSEL;
  pDest->RWC = pSrc->RWC;
  pDest->RWCR = pSrc->RWCR;
  pDest->TWC = pSrc->TWC;
  pDest->TWCR = pSrc->TWCR;
  pDest->MMRDH = pSrc->MMRDH;
  pDest->MMTOP = pSrc->MMTOP;
}


__ramfunc static void spi_configure(ADI_SPI_TypeDef *pSpiRegs)
{
  /* need to ensure SPI is configured for 1 bit mode of operation */
  pSpiRegs->CTL = 0;   /* Disable the SPI */
  pSpiRegs->CLK = 0x3; /* Slow down the clock for the register configuration */
  pSpiRegs->RXCTL = 0x0;
  pSpiRegs->TXCTL = 0x0;
  pSpiRegs->TWC = 0;
  pSpiRegs->RWC = 0;

  /* Now set up SPI for single bit mode of operation */
  pSpiRegs->CTL = 0x3;
}

__ramfunc void spi_send_bytes(ADI_SPI_TypeDef *pSpiRegs, uint8_t * pData, uint32_t nNumBytes)
{
  uint32_t i;

  if((nNumBytes != 0))
  {
    uint32_t txctl = pSpiRegs->TXCTL;
    pSpiRegs->RXCTL = 0;

    pSpiRegs->TWC = nNumBytes;
    pSpiRegs->TXCTL = BITM_SPI_TXCTL_TEN | BITM_SPI_TXCTL_TTI | BITM_SPI_TXCTL_TWCEN;

    for(i=0; i<nNumBytes; i++)
    {
      pSpiRegs->TFIFO = (pData[i]);
      while((pSpiRegs->STAT & BITM_SPI_STAT_TFS) == ENUM_SPI_STAT_TFIFO_FULL);  /* Wait until there is space in the FIFO */
    }

    while((pSpiRegs->STAT & BITM_SPI_STAT_TF) == 0);  /* Wait until finished */
    pSpiRegs->STAT = BITM_SPI_STAT_TF;
    pSpiRegs->TXCTL = txctl & (~BITM_SPI_TXCTL_TEN);
  }
}


__ramfunc void spi_exit_xip(ADI_SPI_TypeDef * pSpiRegs)
{
  /* send Continuous Read Mode Reset (FFh or FFFFh) */
  uint32_t nSlvSelHigh = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00));
  uint32_t nSlvSelLow = ((BITM_SPI_SLVSEL_SSE1 << ADI_SPI2_SSEL_ID) | (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << ADI_SPI2_SSEL_ID)));
  uint8_t spiData[1];

  spiData[0] = 0xFF;

  pSpiRegs->SLVSEL = nSlvSelLow; /* set slave select low */
  pSpiRegs->SLVSEL = nSlvSelLow; /* set slave select low */
  spi_send_bytes(pSpiRegs, &spiData[0], 1);

  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
  pSpiRegs->SLVSEL = nSlvSelHigh; /* set slave select high */
}

__ramfunc void spi_enter_xip(ADI_SPI_TypeDef * pSpiRegs)
{
  pSpiRegs->MMRDH = (pSpiRegs->MMRDH & (~BITM_SPI_MMRDH_MODE)) | XIP_CONFIRMATION_BIT;          /* Setting the correct mode bits */
  pSpiRegs->CTL |= BITM_SPI_CTL_EN; 
  
  /* Dummy read through pointer */
  temp = *pSPI_MEM;
   
  pSpiRegs-> MMRDH |= BITM_SPI_MMRDH_CMDSKIP;           /* Setting command skip */
}